@@ -100,7 +100,7 @@ class _httprequesthandler(httpservermod.basehttprequesthandler):
 
     def log_request(self, code='-', size='-'):
         xheaders = []
-        if util.safehasattr(self, b'headers'):
+        if util.safehasattr(self, 'headers'):
             xheaders = [
                 h for h in self.headers.items() if h[0].startswith('x-')
             ]
@@ -151,6 +151,9 @@ class _httprequesthandler(httpservermod.basehttprequesthandler):
     def do_GET(self):
         self.do_POST()
 
+    def do_HEAD(self):
+        self.do_POST()
+
     def do_hgweb(self):
         self.sent_headers = False
         path, query = _splitURI(self.path)
@@ -211,7 +214,7 @@ class _httprequesthandler(httpservermod.basehttprequesthandler):
         env['wsgi.multithread'] = isinstance(
             self.server, socketserver.ThreadingMixIn
         )
-        if util.safehasattr(socketserver, b'ForkingMixIn'):
+        if util.safehasattr(socketserver, 'ForkingMixIn'):
             env['wsgi.multiprocess'] = isinstance(
                 self.server, socketserver.ForkingMixIn
             )
@@ -246,7 +249,11 @@ class _httprequesthandler(httpservermod.basehttprequesthandler):
             self.send_header(*h)
             if h[0].lower() == 'content-length':
                 self.length = int(h[1])
-        if self.length is None and saved_status[0] != common.HTTP_NOT_MODIFIED:
+        if (
+            self.length is None
+            and saved_status[0] != common.HTTP_NOT_MODIFIED
+            and self.command != 'HEAD'
+        ):
             self._chunked = (
                 not self.close_connection and self.request_version == 'HTTP/1.1'
             )
@@ -337,7 +344,7 @@ try:
     threading.active_count()  # silence pyflakes and bypass demandimport
     _mixin = socketserver.ThreadingMixIn
 except ImportError:
-    if util.safehasattr(os, b"fork"):
+    if util.safehasattr(os, "fork"):
         _mixin = socketserver.ForkingMixIn
     else:
 
